/*
 * motors.c
 *
 *  Created on: 02.02.2014
 *      Author: AndreR
 */

#include "motors.h"

#include <main/ctrl.h>

#include "spi3.h"
#include "hal/exti.h"
#include "constants.h"
#include "util/console.h"
#include "intercom1.h"
#include "intercom_constants.h"
#include "util/boot.h"
#include "util/init_hal.h"
#include "hal/buzzer.h"
#include "ctrl_impl.h"
#include "log_msgs.h"

#include "ch.h"

Motors motors;

static void spi3AboutToSend(SPI3Slave* pSlave, uint8_t slaveID)
{
	(void)pSlave;

	IntercomHeader iHeader;
	iHeader.command = IC_MOTORS_MAIN2MOT;
	iHeader.section = INTERCOM_SECTION_MOTORS;

	MotorsMain2Mot main2Mot;
	main2Mot.bat = ctrl.input.power.voltage;
	main2Mot.cur = ctrl.input.power.current;
	main2Mot.ctrlMode = motors.ctrlMode;

	if(slaveID == SPI3_SLAVE_RIGHT)
	{
		iHeader.address = INTERCOM_ADDRESS_RIGHT;

		memcpy(&main2Mot.mot[MOT_FRONT], &motors.mot[MOT_FR].set, sizeof(MotorSetpoint));
		memcpy(&main2Mot.mot[MOT_REAR], &motors.mot[MOT_RR].set, sizeof(MotorSetpoint));
	}
	else
	{
		iHeader.address = INTERCOM_ADDRESS_LEFT;

		memcpy(&main2Mot.mot[MOT_FRONT], &motors.mot[MOT_FL].set, sizeof(MotorSetpoint));
		memcpy(&main2Mot.mot[MOT_REAR], &motors.mot[MOT_RL].set, sizeof(MotorSetpoint));
	}

	IntercomSend(&iHeader, sizeof(main2Mot), (uint8_t*)&main2Mot);
}

static void motorsCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;

	if(pHeader->command == IC_MOTORS_MOT2MAIN && dataLength >= sizeof(MotorsMot2Main))
	{
		MotorsMot2Main* pMot2Main = (MotorsMot2Main*)pData;

		if(pMot2Main->source == MOT_SOURCE_RIGHT)
		{
			memcpy(&motors.mot[MOT_FR].cur, &pMot2Main->mot[MOT_FRONT], sizeof(MotorFeedback));
			memcpy(&motors.mot[MOT_RR].cur, &pMot2Main->mot[MOT_REAR], sizeof(MotorFeedback));

			motors.lastMotRx[0] = chVTGetSystemTimeX();
		}
		else
		{
			memcpy(&motors.mot[MOT_FL].cur, &pMot2Main->mot[MOT_FRONT], sizeof(MotorFeedback));
			memcpy(&motors.mot[MOT_RL].cur, &pMot2Main->mot[MOT_REAR], sizeof(MotorFeedback));

			motors.lastMotRx[1] = chVTGetSystemTimeX();
		}

		if(motors.updateCb)
			(*motors.updateCb)(pMot2Main->source);
	}
}

void MotorsInit()
{
	motors.ctrlMode = MOT_CTRL_VELOCITY;

	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_INPUT;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOA, GPIO_PIN_10 | GPIO_PIN_12, &gpioInit);
	GPIOInit(GPIOC, GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_8, &gpioInit);

	motors.mot[0].pFaultPort = GPIOA;
	motors.mot[0].faultPin = GPIO_PIN_12;
	motors.mot[1].pFaultPort = GPIOC;
	motors.mot[1].faultPin = GPIO_PIN_1;
	motors.mot[2].pFaultPort = GPIOC;
	motors.mot[2].faultPin = GPIO_PIN_2;
	motors.mot[3].pFaultPort = GPIOC;
	motors.mot[3].faultPin = GPIO_PIN_8;

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &motorsCallback;
	client.section = INTERCOM_SECTION_MOTORS;

	IntercomAddClient(client);

	spi3.aboutToSendCb = &spi3AboutToSend;
}

void MotorsTask(void* params)
{
	(void)params;

	chRegSetThreadName("Motors");

	chThdSleep(MS2ST(2000));

	motors.lastMotRx[0] = chVTGetSystemTimeX();
	motors.lastMotRx[1] = chVTGetSystemTimeX();

	while(1)
	{
		chThdSleepMilliseconds(1);

		for(uint16_t i = 0; i < 4; i++)
		{
			if((motors.mot[i].pFaultPort->IDR & motors.mot[i].faultPin) == 0)
			{
				// fault pin low
				++motors.mot[i].faultCounter;
				if(motors.mot[i].faultCounter == 1)
				{
					motors.mot[i].fault = 1;

					ConsolePrint("Motor fault %hu\r\n", i);
				}
			}
			else
			{
				motors.mot[i].faultCounter = 0;
				motors.mot[i].fault = 0;
			}
		}
	}
}
