/*
 * prog_store_impl.c
 *
 *  Created on: 13.04.2016
 *      Author: AndreR
 */

#include "prog_store_impl.h"
#include "util/prog_store.h"
#include "intercom_constants.h"
#include "string.h"
#include "util/crc.h"
#include "util/flash.h"

#define PROG_SIZE_MAIN (224*1024)
#define PROG_SIZE_KD (92*1024)
#define PROG_SIZE_MOT (92*1024)

#define PROG_BASE_ADDR 0x08080000

#define PROG_INFO_ADDR	(PROG_BASE_ADDR+0)
#define PROG_MAIN_ADDR	(PROG_INFO_ADDR+4096)
#define PROG_LEFT_ADDR	(PROG_MAIN_ADDR+PROG_SIZE_MAIN)
#define PROG_RIGHT_ADDR	(PROG_LEFT_ADDR+PROG_SIZE_MOT)
#define PROG_KD_ADDR	(PROG_RIGHT_ADDR+PROG_SIZE_MOT)


const uint8_t* pProgs[] = {
	(const uint8_t*)PROG_LEFT_ADDR,
	(const uint8_t*)PROG_RIGHT_ADDR,
	(const uint8_t*)PROG_KD_ADDR,
	(const uint8_t*)PROG_MAIN_ADDR,
};

const ProgUpgradeInfo* pInfoFlash = (ProgUpgradeInfo*)PROG_INFO_ADDR;

void ProgStoreImplInit()
{
	ProgStoreAddTarget(INTERCOM_ADDRESS_LEFT, PROG_SIZE_MOT, PROG_STORE_ID_LEFT);
	ProgStoreAddTarget(INTERCOM_ADDRESS_RIGHT, PROG_SIZE_MOT, PROG_STORE_ID_RIGHT);
	ProgStoreAddTarget(INTERCOM_ADDRESS_KD, PROG_SIZE_KD, PROG_STORE_ID_KD);
	ProgStoreAddTarget(INTERCOM_ADDRESS_MAIN, PROG_SIZE_MAIN, PROG_STORE_ID_MAIN);
}

void ProgStoreImplWriteData(uint8_t procId, uint32_t offset, uint8_t* pData, uint32_t size)
{
	FlashProgram((uint32_t)(pProgs[procId]+offset), (uint32_t*)pData, (size+3)/4);
}

void ProgStoreImplGetUpgradeInfo(uint8_t procId, ProgUpgradeInfo* pInfo)
{
	pInfo->update = pInfoFlash[procId].update;
	pInfo->crc = pInfoFlash[procId].crc;
	pInfo->size = pInfoFlash[procId].size;
}

void ProgStoreImplSetUpgradeInfo(uint8_t procId, const ProgUpgradeInfo* pInfo)
{
	FlashProgram((uint32_t)(&pInfoFlash[procId]), (uint32_t*)pInfo, sizeof(ProgUpgradeInfo)/4);
}

void ProgStoreImplReadData(uint8_t procId, uint32_t offset, uint8_t* pBuf, uint32_t size)
{
	memcpy(pBuf, pProgs[procId]+offset, size);
}

uint32_t ProgStoreImplGetChecksum(uint8_t procId, uint32_t offset, uint32_t size)
{
	return CRC32CalcChecksumAlt(pProgs[procId]+offset, size);
}

void ProgStoreImplEraseStorage()
{
	// erase upper 512kB of program memory
	// this will stall execution for around 12s
	FlashErase(0x08080000, 0x08100000);
}
