/*
 * ui.c
 *
 *  Created on: 22.05.2014
 *      Author: AndreR
 */

#include "ui.h"

#include <main/ctrl.h>
#include "util/intercom.h"
#include "intercom_constants.h"
#include "main/network.h"

UI ui;

static void uiCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	switch(pHeader->command)
	{
		case IC_UI_WIFI_BOTID:
		{
			UIWifiBotID* pID = (UIWifiBotID*)pData;

			network.config.botId = pID->id;
			NetworkSaveConfig();
		}
		break;
		case IC_UI_WIFI_SETTINGS:
		{
			UIWifiSettings* pSet = (UIWifiSettings*)pData;

			network.config.speed = pSet->speed;
			network.config.channel = pSet->channel;
			NetworkSaveConfig();
		}
		break;
		case IC_UI_WIFI_SAVE:
		{
			// obsolete
		}
		break;
		case IC_UI_FEATURES:
		{
			uint16_t* pFeatures = (uint16_t*)pData;

			CtrlSetFeatures(*pFeatures);
		}
		break;
	}
}

void UIInit()
{
	chMBObjectInit(&ui.eventQueue, ui.eventQueueData, UI_EVENT_QUEUE_SIZE);

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &uiCallback;
	client.section = INTERCOM_SECTION_UI;

	IntercomAddClient(client);
}

void UITask(void* params)
{
	(void)params;
	msg_t event;

	chRegSetThreadName("UI");

	while(1)
	{
		chMBFetch(&ui.eventQueue, &event, MS2ST(500));

		IntercomHeader iHeader;
		iHeader.address = INTERCOM_ADDRESS_MEDIA;
		iHeader.section = INTERCOM_SECTION_PRESENTER;
		iHeader.command = IC_PRESENTER_CONFIG;

		IntercomSend(&iHeader, sizeof(ConfigNetwork), (uint8_t*)&network.config);

		iHeader.command = IC_PRESENTER_CFG_MISC;

		IntercomSend(&iHeader, sizeof(ConfigMisc), &ctrl.misc);
	}
}
