/*
 * features.c
 *
 *  Created on: 05.10.2015
 *      Author: AndreR
 */

#include "features.h"
#include "commands.h"
#include "ids.h"
#include "intercom_constants.h"
#include "util/intercom_reliable.h"

static const char* featureNames[] = {"Move", "Dribbler", "Straight Kick", "Chip Kick", "Barrier"};

static struct _handles
{
	GHandle hFeatures[SYSTEM_MATCH_FEEDBACK_NUM_FEATURES];
	GHandle hSave;
	GHandle hTop;
} handles;

static int16_t eventHandler(GEvent* pEvent)
{
	if(pEvent->type == GEVENT_GWIN_BUTTON)
	{
		GEventGWinButton* pBtn = (GEventGWinButton*)pEvent;

		if(pBtn->gwin == handles.hSave)
		{
			uint16_t features = 0;
			for(uint8_t i = 0; i < SYSTEM_MATCH_FEEDBACK_NUM_FEATURES; i++)
			{
				if(gwinCheckboxIsChecked(handles.hFeatures[i]))
					features |= (1 << i);
			}

			IntercomHeader iHeader;
			iHeader.address = INTERCOM_ADDRESS_MAIN;
			iHeader.section = INTERCOM_SECTION_UI;
			iHeader.command = IC_UI_FEATURES;

			IntercomReliableSend(&iHeader, sizeof(uint16_t), &features);

			return GUI_WINDOW_STATUS;
		}
	}

	return -1;
}

GHandle FeaturesCreate()
{
    GWidgetInit topContInit = { { 0, 60, 240, 260, FALSE, 0 }, "Features", 0, &eventHandler, 0 };
    GHandle hTop = gwinContainerCreate(0, &topContInit, 0);

	for(uint8_t i = 0; i < SYSTEM_MATCH_FEEDBACK_NUM_FEATURES; i++)
	{
		GWidgetInit boxInit = { { 5, 10+38*i, 110, 20, TRUE, hTop }, featureNames[i], 0, 0, 0 };
		handles.hFeatures[i] = gwinCheckboxCreate(0, &boxInit);
	}

	GWidgetInit saveBtnInit = { { 30, 200, 180, 30, TRUE, hTop }, "Save", 0, 0, 0 };
	handles.hSave = gwinButtonCreate(0, &saveBtnInit);

	handles.hTop = hTop;

    return hTop;
}

void FeaturesSet(uint16_t features)
{
	if(gwinGetVisible(handles.hTop))
		return;

	for(uint8_t i = 0; i < SYSTEM_MATCH_FEEDBACK_NUM_FEATURES; i++)
	{
		uint16_t check = (features >> i) & 0x0001;

		gwinCheckboxCheck(handles.hFeatures[i], check);
	}
}
