/*
 * optical.c
 *
 *  Created on: 17.10.2015
 *      Author: AndreR
 */

#include "optical.h"
#include <stdio.h>

static struct _handles
{
	GHandle hMotion;
	GHandle hDeltaX;
	GHandle hDeltaY;
	GHandle hSQUAL;
	GHandle hPixelSum;
	GHandle hMaxPixel;
	GHandle hMinPixel;
	GHandle hShutter;
	GHandle hFramePeriod;
} handles;

static GHandle createLabel(coord_t x, coord_t y, coord_t w, coord_t h, GHandle parent, const char* pText)
{
	GWidgetInit deviceLabelInit = { { x, y, w, h, TRUE, parent }, pText, 0, 0, 0 };
	return gwinLabelCreate(0, &deviceLabelInit);
}

GHandle OpticalCreate()
{
    GWidgetInit wi = { { 0, 60, 240, 260, FALSE, 0 }, "Optical", 0, 0, 0 };
    GHandle hTop = gwinContainerCreate(0, &wi, 0);

    font_t curFont = gwinGetDefaultFont();
    gwinSetDefaultFont(gdispOpenFont("DejaVuSans10"));

    createLabel(180, 5, 50, 15, hTop, "Motion:");
    handles.hMotion = createLabel(210, 5, 60, 15, hTop, "42");

    createLabel(5, 5, 70, 15, hTop, "DeltaX:");
    handles.hDeltaX = createLabel(80, 5, 160, 15, hTop, "dev");

    createLabel(5, 20, 70, 15, hTop, "DeltaY:");
    handles.hDeltaY = createLabel(80, 20, 160, 15, hTop, "?");

    createLabel(5, 35, 70, 15, hTop, "SQUAL:");
    handles.hSQUAL = createLabel(80, 35, 160, 15, hTop, "prod");

    createLabel(5, 50, 70, 15, hTop, "PixelSum:");
    handles.hPixelSum = createLabel(80, 50, 160, 15, hTop, "012345");

    createLabel(5, 65, 70, 15, hTop, "Max Pixel:");
    handles.hMaxPixel = createLabel(80, 65, 160, 15, hTop, "42 MB");

    createLabel(5, 80, 70, 15, hTop, "Min Pixel:");
    handles.hMinPixel = createLabel(80, 80, 160, 15, hTop, "43 MB");

    createLabel(5, 95, 70, 15, hTop, "Shutter:");
    handles.hShutter = createLabel(80, 95, 160, 15, hTop, "16k");

    createLabel(5, 125, 70, 15, hTop, "Frame Rate:");
    handles.hFramePeriod = createLabel(80, 125, 160, 15, hTop, "N/A");

    gwinSetDefaultFont(curFont);

    return hTop;
}

void OpticalUpdate(ADNS9800Motion* pMot)
{
	static char deltaX[10];
	static char deltaY[10];
	static char squal[10];
	static char pixelSum[10];
	static char minPixel[10];
	static char maxPixel[10];
	static char shutter[10];
	static char frame[10];

	snprintf(deltaX, 10, "%hd", pMot->deltaX);
	gwinSetText(handles.hDeltaX, deltaX, FALSE);

	snprintf(deltaY, 10, "%hd", pMot->deltaY);
	gwinSetText(handles.hDeltaY, deltaY, FALSE);

	snprintf(squal, 10, "%hu", (uint16_t)pMot->squal);
	gwinSetText(handles.hSQUAL, squal, FALSE);

	snprintf(pixelSum, 10, "%hu", (uint16_t)pMot->pixelSum);
	gwinSetText(handles.hPixelSum, pixelSum, FALSE);

	snprintf(minPixel, 10, "%hu", (uint16_t)pMot->minPixel);
	gwinSetText(handles.hMinPixel, minPixel, FALSE);

	snprintf(maxPixel, 10, "%hu", (uint16_t)pMot->maxPixel);
	gwinSetText(handles.hMaxPixel, maxPixel, FALSE);

	uint32_t shutterNs = pMot->shutter*20;
	snprintf(shutter, 10, "%u", shutterNs);
	gwinSetText(handles.hShutter, shutter, FALSE);

	uint32_t frameRate = 50000000/pMot->framePeriod;
	snprintf(frame, 10, "%u", frameRate);
	gwinSetText(handles.hFramePeriod, frame, FALSE);
}
