/*
 * test_drive.c
 *
 *  Created on: 14.12.2015
 *      Author: AndreR <andre@ryll.cc>
 */

#include "test_drive.h"
#include "styles.h"
#include "util/intercom.h"
#include "util/intercom_reliable.h"
#include "../test.h"
#include <stdio.h>
#include <math.h>

static const char* pMotors[] = {"FR", "FL", "RL", "RR"};

static struct _handles
{
	GHandle hOverviewCont;
	GHandle hBtnTestAll;

	struct
	{
		GHandle hBtnMotor;
		GHandle hMotCont;
		GHandle hBtnBack;
		GHandle hBtnTest;
		GLabelObject* hPower;
		GLabelObject* hHall;
		GLabelObject* hEnc;
		GLabelObject* hDyn;

		struct
		{
			GLabelObject* hMain[3];
			GLabelObject* hMot[3];
			GLabelObject* hR[3];
			char main[3][8];
			char mot[3][8];
			char r[3][8];
		} pwrUnit;

		GLabelObject* hHallRow[6];
		char hall[6][16];

		GLabelObject* hDynamicsRow[2];
		char dyn[2][36];

		GLabelObject* hEncoderRow;
		char enc[16];
	} mot[4];
} handles;

static int16_t eventHandler(GEvent* pEvent)
{
	if(pEvent->type == GEVENT_GWIN_BUTTON)
	{
		GEventGWinButton* pBtn = (GEventGWinButton*)pEvent;

		if(pBtn->gwin == handles.hBtnTestAll)
		{
			for(uint8_t i = 0; i < 4; i++)
			{
				handles.mot[i].hPower->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hPower->w.g, "-", FALSE);

				handles.mot[i].hHall->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hHall->w.g, "-", FALSE);

				handles.mot[i].hEnc->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hEnc->w.g, "-", FALSE);

				handles.mot[i].hDyn->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hDyn->w.g, "-", FALSE);

				gwinSetEnabled(handles.mot[i].hBtnTest, FALSE);

				TestScheduleMotorTest(IC_TEST_MOT_POWER, i);
				TestScheduleMotorTest(IC_TEST_MOT_HALL, i);
				TestScheduleMotorTest(IC_TEST_MOT_ENC, i);
				TestScheduleMotorTest(IC_TEST_MOT_DYN, i);
			}

			gwinSetEnabled(handles.hBtnTestAll, FALSE);
		}

		for(uint8_t i = 0; i < 4; i++)
		{
			if(pBtn->gwin == handles.mot[i].hBtnMotor)
			{
				gwinSetVisible(handles.hOverviewCont, FALSE);
				gwinSetVisible(handles.mot[i].hMotCont, TRUE);
			}

			if(pBtn->gwin == handles.mot[i].hBtnBack)
			{
				gwinSetVisible(handles.mot[i].hMotCont, FALSE);
				gwinSetVisible(handles.hOverviewCont, TRUE);
			}

			if(pBtn->gwin == handles.mot[i].hBtnTest)
			{
				TestScheduleMotorTest(IC_TEST_MOT_POWER, i);
				TestScheduleMotorTest(IC_TEST_MOT_HALL, i);
				TestScheduleMotorTest(IC_TEST_MOT_ENC, i);
				TestScheduleMotorTest(IC_TEST_MOT_DYN, i);

				gwinSetEnabled(handles.mot[i].hBtnTest, FALSE);
			}
		}
	}

	return -1;
}

void TestDriveProgress(TestProgress* pProgress)
{
	static char buf[8];

	snprintf(buf, 8, "%3hu%%", (uint16_t)pProgress->percentComplete);

	switch(pProgress->testId)
	{
		case IC_TEST_MOT_POWER:
		{
			gwinSetText(&handles.mot[pProgress->motorId].hPower->w.g, buf, FALSE);
		}
		break;
		case IC_TEST_MOT_HALL:
		{
			gwinSetText(&handles.mot[pProgress->motorId].hHall->w.g, buf, FALSE);
		}
		break;
		case IC_TEST_MOT_DYN:
		{
			gwinSetText(&handles.mot[pProgress->motorId].hDyn->w.g, buf, FALSE);
		}
		break;
		case IC_TEST_MOT_ENC:
		{
			gwinSetText(&handles.mot[pProgress->motorId].hEnc->w.g, buf, FALSE);
		}
		break;
	}
}

void TestDrivePowerUnitResult(TestMotorPowerUnitResult* pResult)
{
	for(uint8_t i = 0; i < 3; i++)
	{
		snprintf(handles.mot[pResult->motorId].pwrUnit.main[i], 8, "%.3f", pResult->mainCur[i]);
		snprintf(handles.mot[pResult->motorId].pwrUnit.mot[i], 8, "%.3f", pResult->motCur[i]);
		snprintf(handles.mot[pResult->motorId].pwrUnit.r[i], 8, "%.3f", pResult->R[i]);

		gwinSetText(&handles.mot[pResult->motorId].pwrUnit.hMain[i]->w.g, handles.mot[pResult->motorId].pwrUnit.main[i], FALSE);
		gwinSetText(&handles.mot[pResult->motorId].pwrUnit.hMot[i]->w.g, handles.mot[pResult->motorId].pwrUnit.mot[i], FALSE);
		gwinSetText(&handles.mot[pResult->motorId].pwrUnit.hR[i]->w.g, handles.mot[pResult->motorId].pwrUnit.r[i], FALSE);

		handles.mot[pResult->motorId].pwrUnit.hR[i]->w.pstyle = &BlackWidgetStyle;
	}

	uint8_t warn = 0;
	uint8_t fail = 0;

	for(uint8_t i = 0; i < 3; i++)
	{
		if(pResult->R[i] < 0.45f || pResult->R[i] > 0.75f)
		{
			warn = 1;
			handles.mot[pResult->motorId].pwrUnit.hR[i]->w.pstyle = &YellowTextStyle;
		}

		if(pResult->R[i] < 0.2f || pResult->R[i] > 1.0f)
		{
			fail = 1;
			handles.mot[pResult->motorId].pwrUnit.hR[i]->w.pstyle = &RedTextStyle;
		}
	}

	if(fail)
	{
		gwinSetText(&handles.mot[pResult->motorId].hPower->w.g, "FAIL", FALSE);
		handles.mot[pResult->motorId].hPower->w.pstyle = &RedTextStyle;
	}
	else if(warn)
	{
		gwinSetText(&handles.mot[pResult->motorId].hPower->w.g, "WARN", FALSE);
		handles.mot[pResult->motorId].hPower->w.pstyle = &YellowTextStyle;
	}
	else
	{
		gwinSetText(&handles.mot[pResult->motorId].hPower->w.g, "OK", FALSE);
		handles.mot[pResult->motorId].hPower->w.pstyle = &GreenTextStyle;
	}
}

void TestDriveHallResult(TestMotorHallResult* pResult)
{
	uint8_t warn = 0;
	uint8_t fail = 0;

	for(uint8_t i = 0; i < 6; i++)
	{
		snprintf(handles.mot[pResult->motorId].hall[i], 16, "%3hd - %3hd - %3hd",
				pResult->pos[i].fwd, pResult->pos[i].ref, pResult->pos[i].rev);

		gwinSetText(&handles.mot[pResult->motorId].hHallRow[i]->w.g, handles.mot[pResult->motorId].hall[i], FALSE);
		handles.mot[pResult->motorId].hHallRow[i]->w.pstyle = &BlackWidgetStyle;

		int16_t diffFwd = pResult->pos[i].ref - pResult->pos[i].fwd;
		if(i == 0)
			diffFwd += 360;

		int16_t diffRev = pResult->pos[i].rev - pResult->pos[i].ref;

		if(diffFwd > 35 || diffRev > 35)
		{
			warn = 1;
			handles.mot[pResult->motorId].hHallRow[i]->w.pstyle = &YellowTextStyle;
		}

		if(diffFwd > 40 || diffRev > 40)
		{
			fail = 1;
			handles.mot[pResult->motorId].hHallRow[i]->w.pstyle = &RedTextStyle;
		}
	}

	if(fail)
	{
		gwinSetText(&handles.mot[pResult->motorId].hHall->w.g, "FAIL", FALSE);
		handles.mot[pResult->motorId].hHall->w.pstyle = &RedTextStyle;
	}
	else if(warn)
	{
		gwinSetText(&handles.mot[pResult->motorId].hHall->w.g, "WARN", FALSE);
		handles.mot[pResult->motorId].hHall->w.pstyle = &YellowTextStyle;
	}
	else
	{
		gwinSetText(&handles.mot[pResult->motorId].hHall->w.g, "OK", FALSE);
		handles.mot[pResult->motorId].hHall->w.pstyle = &GreenTextStyle;
	}
}

void TestDriveEncoderResult(TestMotorEncoderResult* pResult)
{
	uint8_t warn = 0;
	uint8_t fail = 0;

	snprintf(handles.mot[pResult->motorId].enc, 16, "%5hu (%5hu)", pResult->measured, pResult->expected);
	gwinSetText(&handles.mot[pResult->motorId].hEncoderRow->w.g, handles.mot[pResult->motorId].enc, FALSE);
	handles.mot[pResult->motorId].hEncoderRow->w.pstyle = &BlackWidgetStyle;

	float deviation = fabsf(((float)pResult->measured)/((float)pResult->expected)*100.0f-100.0f);
	if(deviation > 0.5f)
	{
		warn = 1;
		handles.mot[pResult->motorId].hEncoderRow->w.pstyle = &YellowTextStyle;
	}

	if(deviation > 5.0f)
	{
		fail = 1;
		handles.mot[pResult->motorId].hEncoderRow->w.pstyle = &RedTextStyle;
	}

	if(fail)
	{
		gwinSetText(&handles.mot[pResult->motorId].hEnc->w.g, "FAIL", FALSE);
		handles.mot[pResult->motorId].hEnc->w.pstyle = &RedTextStyle;
	}
	else if(warn)
	{
		gwinSetText(&handles.mot[pResult->motorId].hEnc->w.g, "WARN", FALSE);
		handles.mot[pResult->motorId].hEnc->w.pstyle = &YellowTextStyle;
	}
	else
	{
		gwinSetText(&handles.mot[pResult->motorId].hEnc->w.g, "OK", FALSE);
		handles.mot[pResult->motorId].hEnc->w.pstyle = &GreenTextStyle;
	}
}

void TestDriveDynamicResult(TestMotorDynamicsResult* pResult)
{
	uint8_t warn = 0;
	uint8_t fail = 0;

	snprintf(handles.mot[pResult->motorId].dyn[0], 36, "R: %.3f (1.2), k: %.2f (25.5)", pResult->R, pResult->k*1000.0f);
	snprintf(handles.mot[pResult->motorId].dyn[1], 36, "I: %.3f (0.2), v: %.1f (320)", pResult->cur, pResult->speed);

	gwinSetText(&handles.mot[pResult->motorId].hDynamicsRow[0]->w.g, handles.mot[pResult->motorId].dyn[0], FALSE);
	gwinSetText(&handles.mot[pResult->motorId].hDynamicsRow[1]->w.g, handles.mot[pResult->motorId].dyn[1], FALSE);

	handles.mot[pResult->motorId].hDynamicsRow[0]->w.pstyle = &BlackWidgetStyle;
	handles.mot[pResult->motorId].hDynamicsRow[1]->w.pstyle = &BlackWidgetStyle;

	if(pResult->R < 0.9f || pResult->R > 1.5f)
	{
		warn = 1;
		handles.mot[pResult->motorId].hDynamicsRow[0]->w.pstyle = &YellowTextStyle;
	}

	if(pResult->R < 0.6f || pResult->R > 1.8f)
	{
		fail = 1;
		handles.mot[pResult->motorId].hDynamicsRow[0]->w.pstyle = &RedTextStyle;
	}

	if(pResult->k < 0.023f || pResult->k > 0.03f)
	{
		fail = 1;
		handles.mot[pResult->motorId].hDynamicsRow[0]->w.pstyle = &RedTextStyle;
	}

	if(pResult->cur < 0.125f || pResult->cur > 0.3f)
	{
		warn = 1;
		handles.mot[pResult->motorId].hDynamicsRow[1]->w.pstyle = &YellowTextStyle;
	}

	if(pResult->cur < 0.08f || pResult->cur > 0.6f)
	{
		fail = 1;
		handles.mot[pResult->motorId].hDynamicsRow[1]->w.pstyle = &RedTextStyle;
	}

	if(pResult->speed < 300 || pResult->speed > 340)
	{
		warn = 1;
		handles.mot[pResult->motorId].hDynamicsRow[1]->w.pstyle = &YellowTextStyle;
	}

	if(pResult->speed < 200 || pResult->speed > 400)
	{
		fail = 1;
		handles.mot[pResult->motorId].hDynamicsRow[1]->w.pstyle = &RedTextStyle;
	}

	if(fail)
	{
		gwinSetText(&handles.mot[pResult->motorId].hDyn->w.g, "FAIL", FALSE);
		handles.mot[pResult->motorId].hDyn->w.pstyle = &RedTextStyle;
	}
	else if(warn)
	{
		gwinSetText(&handles.mot[pResult->motorId].hDyn->w.g, "WARN", FALSE);
		handles.mot[pResult->motorId].hDyn->w.pstyle = &YellowTextStyle;
	}
	else
	{
		gwinSetText(&handles.mot[pResult->motorId].hDyn->w.g, "OK", FALSE);
		handles.mot[pResult->motorId].hDyn->w.pstyle = &GreenTextStyle;
	}

	gwinSetEnabled(handles.mot[pResult->motorId].hBtnTest, TRUE);

	if(pResult->motorId == 3)
		gwinSetEnabled(handles.hBtnTestAll, TRUE);
}

GHandle TestDriveCreate()
{
    GWidgetInit wi = { { 0, 60, 240, 260, FALSE, 0 }, "Drives", 0, &eventHandler, 0 };
    GHandle hTop = gwinContainerCreate(0, &wi, 0);

    for(uint8_t i = 0; i < 4; i++)
    {
		GWidgetInit motContInit = { { 0, 0, 240, 260, FALSE, hTop }, 0, 0, 0, 0 };
		handles.mot[i].hMotCont = gwinContainerCreate(0, &motContInit, 0);

		GWidgetInit backBtnInit = { { 140, 0, 80, 25, TRUE, handles.mot[i].hMotCont }, pMotors[i], 0, 0, 0 };
		handles.mot[i].hBtnBack = gwinButtonCreate(0, &backBtnInit);

		// Power Unit
		GWidgetInit powerUnitContInit = { { 5, 5, 110, 160, TRUE, handles.mot[i].hMotCont }, 0, 0, 0, 0 };
		GHandle hPwrUnit = gwinContainerCreate(0, &powerUnitContInit, 0);

		GWidgetInit powerUnitInit = { { 0, 0, 110, 20, TRUE, hPwrUnit }, "-- Power Unit --", 0, 0, 0 };
		gwinLabelCreate(0, &powerUnitInit);

		GWidgetInit powerUnitMainInit = { { 0, 20, 60, 20, TRUE, hPwrUnit }, "Main", 0, 0, 0 };
		gwinLabelCreate(0, &powerUnitMainInit);

		GWidgetInit powerUnitMotInit = { { 60, 20, 60, 20, TRUE, hPwrUnit }, "Mot", 0, 0, 0 };
		gwinLabelCreate(0, &powerUnitMotInit);

		GWidgetInit powerUnitR1Init = { { 30, 100, 30, 20, TRUE, hPwrUnit }, "R1", 0, 0, 0 };
		gwinLabelCreate(0, &powerUnitR1Init);

		GWidgetInit powerUnitR2Init = { { 30, 120, 30, 20, TRUE, hPwrUnit }, "R2", 0, 0, 0 };
		gwinLabelCreate(0, &powerUnitR2Init);

		GWidgetInit powerUnitR3Init = { { 30, 140, 30, 20, TRUE, hPwrUnit }, "R3", 0, 0, 0 };
		gwinLabelCreate(0, &powerUnitR3Init);

		for(uint8_t j = 0; j < 3; j++)
		{
			GWidgetInit powerUnitMain1Init = { { 0, 40+j*20, 60, 20, TRUE, hPwrUnit }, "-", 0, 0, 0 };
			handles.mot[i].pwrUnit.hMain[j] = (GLabelObject*)gwinLabelCreate(0, &powerUnitMain1Init);

			GWidgetInit powerUnitMot1Init = { { 60, 40+j*20, 60, 20, TRUE, hPwrUnit }, "-", 0, 0, 0 };
			handles.mot[i].pwrUnit.hMot[j] = (GLabelObject*)gwinLabelCreate(0, &powerUnitMot1Init);

			GWidgetInit powerUnitRInit = { { 60, 100+j*20, 60, 20, TRUE, hPwrUnit }, "-", 0, 0, 0 };
			handles.mot[i].pwrUnit.hR[j] = (GLabelObject*)gwinLabelCreate(0, &powerUnitRInit);
		}

		// Hall Sensors
		GWidgetInit hallContInit = { { 125, 25, 110, 160, TRUE, handles.mot[i].hMotCont }, 0, 0, 0, 0 };
		GHandle hHall = gwinContainerCreate(0, &hallContInit, 0);

		GWidgetInit hallInit = { { 0, 0, 110, 20, TRUE, hHall }, "- Hall Sensors -", 0, 0, 0 };
		gwinLabelCreate(0, &hallInit);

		for(uint8_t j = 0; j < 6; j++)
		{
			GWidgetInit hallRowInit = { { 0, 20+j*20, 110, 20, TRUE, hHall }, "  x -   x -   x", 0, 0, 0 };
			handles.mot[i].hHallRow[j] = (GLabelObject*)gwinLabelCreate(0, &hallRowInit);
		}

		// Encoders
		GWidgetInit encContInit = { { 5, 220, 110, 40, TRUE, handles.mot[i].hMotCont }, 0, 0, 0, 0 };
		GHandle hEnc = gwinContainerCreate(0, &encContInit, 0);

		GWidgetInit encInit = { { 0, 0, 110, 20, TRUE, hEnc }, "-- Encoders --", 0, 0, 0 };
		gwinLabelCreate(0, &encInit);

		GWidgetInit encResInit = { { 0, 20, 110, 20, TRUE, hEnc }, "-", 0, 0, 0 };
		handles.mot[i].hEncoderRow = (GLabelObject*)gwinLabelCreate(0, &encResInit);

		// Dynamics
		GWidgetInit dynContInit = { { 5, 160, 230, 60, TRUE, handles.mot[i].hMotCont }, 0, 0, 0, 0 };
		GHandle hDyn = gwinContainerCreate(0, &dynContInit, 0);

		GWidgetInit dynInit = { { 0, 0, 230, 20, TRUE, hDyn }, "-- Dynamics --", 0, 0, 0 };
		gwinLabelCreate(0, &dynInit);

		GWidgetInit dynRow1Init = { { 0, 20, 230, 20, TRUE, hDyn }, "-", 0, 0, 0 };
		handles.mot[i].hDynamicsRow[0] = (GLabelObject*)gwinLabelCreate(0, &dynRow1Init);

		GWidgetInit dynRow2Init = { { 0, 40, 230, 20, TRUE, hDyn }, "-", 0, 0, 0 };
		handles.mot[i].hDynamicsRow[1] = (GLabelObject*)gwinLabelCreate(0, &dynRow2Init);

		GWidgetInit testBtnInit = { { 125, 225, 110, 30, TRUE, handles.mot[i].hMotCont }, "Test", 0, 0, 0 };
		handles.mot[i].hBtnTest = gwinButtonCreate(0, &testBtnInit);
    }

    // Container for current settings
    GWidgetInit overviewContInit = { { 0, 0, 240, 260, TRUE, hTop }, 0, 0, 0, 0 };
    handles.hOverviewCont = gwinContainerCreate(0, &overviewContInit, 0);

    for(uint8_t i = 0; i < 4; i++)
    {
		GWidgetInit motBtnInit = { { 60+i*45, 5, 40, 40, TRUE, handles.hOverviewCont }, pMotors[i], 0, 0, 0 };
		handles.mot[i].hBtnMotor = gwinButtonCreate(0, &motBtnInit);

	    GWidgetInit powerUnitInit = { { 65+i*45, 55, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hPower = (GLabelObject*)gwinLabelCreate(0, &powerUnitInit);

	    GWidgetInit hallInit = { { 65+i*45, 95, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hHall = (GLabelObject*)gwinLabelCreate(0, &hallInit);

	    GWidgetInit encInit = { { 65+i*45, 135, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hEnc = (GLabelObject*)gwinLabelCreate(0, &encInit);

	    GWidgetInit dynInit = { { 65+i*45, 175, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hDyn = (GLabelObject*)gwinLabelCreate(0, &dynInit);
    }

    GWidgetInit powerUnitLabelInit = { { 5, 55, 55, 16, TRUE, handles.hOverviewCont }, "Power", 0, 0, 0 };
    gwinLabelCreate(0, &powerUnitLabelInit);
    GWidgetInit powerUnitLabelInit2 = { { 5, 71, 55, 16, TRUE, handles.hOverviewCont }, "Unit", 0, 0, 0 };
    gwinLabelCreate(0, &powerUnitLabelInit2);

    GWidgetInit hallLabelInit = { { 5, 95, 55, 16, TRUE, handles.hOverviewCont }, "Hall", 0, 0, 0 };
    gwinLabelCreate(0, &hallLabelInit);
    GWidgetInit hallLabelInit2 = { { 5, 111, 55, 16, TRUE, handles.hOverviewCont }, "Sensors", 0, 0, 0 };
    gwinLabelCreate(0, &hallLabelInit2);

    GWidgetInit encoderLabelInit = { { 5, 135, 55, 35, TRUE, handles.hOverviewCont }, "Encoder", 0, 0, 0 };
    gwinLabelCreate(0, &encoderLabelInit);

    GWidgetInit dynamicsLabelInit = { { 5, 175, 55, 35, TRUE, handles.hOverviewCont }, "Dynamic", 0, 0, 0 };
    gwinLabelCreate(0, &dynamicsLabelInit);

	GWidgetInit testAllBtnInit = { { 5, 220, 230, 40, TRUE, handles.hOverviewCont }, "Test All", 0, 0, 0 };
	handles.hBtnTestAll = gwinButtonCreate(0, &testAllBtnInit);

    return hTop;
}
