#include "ch.h"

#include "util/boot.h"
#include "util/flash_fs.h"
#include "util/crc.h"
#include "util/init_hal.h"
#include "util/network_print.h"
#include "util/sys_time.h"
#include "util/intercom_reliable.h"
#include "util/prog_client.h"
#include "util/config.h"

#include "led.h"
#include "spi1.h"
#include "intercom1.h"
#include "cli.h"
#include "drive.h"
#include "motors.h"
#include "test.h"
#include "cpu_load.h"

static THD_WORKING_AREA(waSPI1, 512);
static THD_WORKING_AREA(waIntercom, 512);
#ifdef ENV_RUN
static THD_WORKING_AREA(swaCLI, 8192);
static THD_WORKING_AREA(_ccm_waDrive, 512);
static THD_WORKING_AREA(waConfig, 512);
static THD_WORKING_AREA(waTest, 4096);
#endif

int main()
{
#ifdef ENV_BOOT
	if(!BootIsBootloaderSelected())
		BootJumpToApplication();

	BootSetBootloaderSelected(0);
#endif

	RCC->AHBENR |= RCC_AHBENR_DMA1EN | RCC_AHBENR_DMA2EN | RCC_AHBENR_GPIOAEN
			| RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN | RCC_AHBENR_GPIOFEN;
	RCC->APB2ENR |= RCC_APB2ENR_SYSCFGEN;

	SCB->SHCSR |= SCB_SHCSR_USGFAULTENA_Msk | SCB_SHCSR_BUSFAULTENA_Msk | SCB_SHCSR_MEMFAULTENA_Msk;

	SystemClockInitData clkInit;
	clkInit.pllMul = 12;	// 6MHz*12 = 72MHz
	clkInit.HSEBypass = 1;
	clkInit.APB1Div = 2;	// = 36MHz
	clkInit.APB2Div = 1;	// = 72MHz
	clkInit.flashLatency = 2;
	clkInit.sysTickFreq = CH_CFG_ST_FREQUENCY;
	SystemClockInit(&clkInit, 72000000UL);

	// set vector table offset
#ifdef ENV_BOOT
	SCB->VTOR = 0;
#else
	SCB->VTOR = BOOT_APP_OFFSET;
#endif

	chSysInit();

/*	// Configure and start Independent watchdog
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
	IWDG_SetPrescaler(IWDG_Prescaler_8);
	IWDG_SetReload(2500);	// approx. 500ms
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);
*/

	// -- HAL
	LEDInit();
	FlashFSInit(0x0801F800, 0x800);	// 2k
	ConfigInit(&ConfigOutputIntercom, INTERCOM_ADDRESS_MEDIA);
	CRCInit();
	SPI1Init();
	Intercom1Init();
	IntercomReliableInit();
	SysTimeInitCorrection(500);
	ProgClientInit();
	ConfigIntercomInit();
#ifdef ENV_RUN
	CLIInit();
	NetworkPrintInitIntercom();
	DriveInit();
	MotorsInit();
	TestInit();
#endif

	// -- Threads
	chThdCreateStatic(waIntercom, sizeof(waIntercom), NORMALPRIO-1, IntercomTask, 0);
	chThdCreateStatic(waSPI1, sizeof(waSPI1), NORMALPRIO+2, SPI1Task, 0);
#ifdef ENV_RUN
	chThdCreateStatic(swaCLI, sizeof(swaCLI), NORMALPRIO-2, CLITask, 0);
	chThdCreateStatic(_ccm_waDrive, sizeof(_ccm_waDrive), NORMALPRIO+40, DriveTask, 0);
	chThdCreateStatic(waConfig, sizeof(waConfig), NORMALPRIO-8, ConfigTask, 0);
	chThdCreateStatic(waTest, sizeof(waTest), NORMALPRIO-20, TestTask, 0);
#endif

	chThdSleepSeconds(2);

	chThdSetPriority(HIGHPRIO-1);

	while(1)
	{
		chThdSleepMilliseconds(10);

		if(chVTTimeElapsedSinceX(motors.lastCmdRx) > MS2ST(500))
		{
			DriveSetMode(pFront, DRIVE_MODE_DISABLE);
			DriveSetMode(pRear, DRIVE_MODE_DISABLE);

			while(1)
			{
				LEDToggle(LED_GREEN);
				chThdSleepMilliseconds(50);
			}
		}

#ifdef ENV_RUN
		uint32_t time = SysTimeUSec()%1000000;

#ifdef MOT_LEFT
		if(time > 200000 && time < 400000)
#else
		if(time > 800000)
#endif
#else
		uint32_t time = SysTimeUSec()%500000;

#ifdef MOT_LEFT
		if(time > 100000 && time < 200000)
#else
		if(time > 400000)
#endif
#endif
			LEDMustOn(LED_GREEN);
		else
			LEDMustOff(LED_GREEN);

		CPULoadUpdateUsage();
	}

	return 0;
}
