/*
 * prog_source_wifi.c
 *
 *  Created on: 02.04.2015
 *      Author: AndreR
 */

#include "prog_source_wifi.h"
#include "util/crc.h"
#include "util/console.h"
#include "commands.h"
#include "intercom_constants.h"
#include "util/intercom.h"
#include "util/log.h"
#include "main/network.h"

#include <string.h>

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size);
static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr);
static void sizeFunc(uint8_t procId);
static void open();
static void close();

ProgSourceWifi progSourceWifi = {.wifi = {&dataFunc, &sizeFunc, &crcFunc, &open, &close, 40, 56}};

void ProgSourceWifiNetworkInput(PacketHeader* pHeader, uint16_t dataLength, uint8_t* pData)
{
	if(pHeader->section != SECTION_BOOTLOADER)
		return;

	switch(pHeader->cmd)
	{
		case CMD_BOOTLOADER_SIZE:
		{
			BootloaderSize* pSize = (BootloaderSize*)pData;

			if(pSize->size == 0xFFFFFFFF)
				ProgStoreProvideError(pSize->procId, PROG_STORE_ERROR_NO_MEDIA);
			else
				ProgStoreProvideSize(pSize->procId, pSize->size);
		}
		break;
		case CMD_BOOTLOADER_CRC32:
		{
			BootloaderCRC32* pCRC = (BootloaderCRC32*)pData;

			ProgStoreProvideCRC32(pCRC->procId, pCRC->startAddr, pCRC->endAddr, pCRC->crc);
		}
		break;
		case CMD_BOOTLOADER_DATA:
		{
			BootloaderData* pBData = (BootloaderData*)pData;
			pData += sizeof(BootloaderData);
			dataLength -= sizeof(BootloaderData);

			ProgStoreProvideData(pBData->procId, pBData->offset, pData, dataLength);
		}
		break;
	}
}

void ProgSourceWifiInit()
{
	ProgStoreAddSource(&progSourceWifi.wifi);
}

static void open()
{
}

static void close()
{
}

static void dataFunc(uint8_t procId, uint32_t offset, uint32_t size)
{
	PacketHeader header;
	BootloaderRequestData req;

	header.section = SECTION_BOOTLOADER;
	header.cmd = CMD_BOOTLOADER_REQUEST_DATA;

	req.procId = procId;
	req.offset = offset;
	req.size = size;

	NetworkSendPacket(&header, &req, sizeof(BootloaderRequestData));
}

static void sizeFunc(uint8_t procId)
{
	PacketHeader header;
	BootloaderRequestSize req;

	header.section = SECTION_BOOTLOADER;
	header.cmd = CMD_BOOTLOADER_REQUEST_SIZE;

	req.procId = procId;

	NetworkSendPacket(&header, &req, sizeof(BootloaderRequestSize));
}

static void crcFunc(uint8_t procId, uint32_t startAddr, uint32_t endAddr)
{
	PacketHeader header;
	BootloaderRequestCRC32 req;

	header.section = SECTION_BOOTLOADER;
	header.cmd = CMD_BOOTLOADER_REQUEST_CRC32;

	req.procId = procId;
	req.startAddr = startAddr;
	req.endAddr = endAddr;

	NetworkSendPacket(&header, &req, sizeof(BootloaderRequestCRC32));
}
