/*
 * skill_basics.c
 *
 *  Created on: 24.06.2015
 *      Author: AndreR
 */

#include <main/ctrl.h>
#include <main/skill_basics.h>

#define GLOBAL_POS_MAX_VEL_XY	5.0f
#define GLOBAL_POS_MAX_VEL_W	30.0f
#define GLOBAL_POS_MAX_ACC_XY	10.0f
#define GLOBAL_POS_MAX_ACC_W	100.0f

#define LOCAL_VEL_MAX_ACC_XY	10.0f
#define LOCAL_VEL_MAX_ACC_W		100.0f
#define LOCAL_VEL_MAX_JERK_XY	100.0f
#define LOCAL_VEL_MAX_JERK_W	1000.0f

static void localVelRun(void* pData, SkillOutput* pOutput);
static void globalVelRun(void* pData, SkillOutput* pOutput);
static void globalPosRun(void* pData, SkillOutput* pOutput);
static void wheelVelRun(void* pData, SkillOutput* pOutput);

SkillInstance skillLocalVel = { 0, &localVelRun, 0 };
SkillInstance skillGlobalVel = { 0, &globalVelRun, 0 };
SkillInstance skillGlobalPos = { 0, &globalPosRun, 0 };
SkillInstance skillWheelVel = { 0, &wheelVelRun, 0 };

typedef struct PACKED _XYWMapping
{
	int16_t x;
	int16_t y;
	int16_t w;
} XYWMapping;

typedef struct PACKED _GlobalPosInput
{
	int16_t xyw[3];

	uint8_t velMaxXY;
	uint8_t velMaxW;
	uint8_t accMaxXY;
	uint8_t accMaxW;
} GlobalPosInput;

typedef struct PACKED _LocalVelInput
{
	int16_t xyw[3];

	uint8_t accMaxXY;
	uint8_t accMaxW;
	uint8_t jerkMaxXY;
	uint8_t jerkMaxW;
} VelInput;

static void localVelRun(void* pData, SkillOutput* pOutput)
{
	VelInput* pVel = (VelInput*)pData;

	pOutput->xyw[0] = pVel->xyw[0]*0.001f;
	pOutput->xyw[1] = pVel->xyw[1]*0.001f;
	pOutput->xyw[2] = pVel->xyw[2]*0.001f;

	if(pVel->accMaxXY == 0)
		pVel->accMaxXY = 1;

	if(pVel->accMaxW == 0)
		pVel->accMaxW = 1;

	if(pVel->jerkMaxXY == 0)
		pVel->jerkMaxXY = 1;

	if(pVel->jerkMaxW == 0)
		pVel->jerkMaxW = 1;

	pOutput->limits.accMaxXY = ((float)pVel->accMaxXY)*(LOCAL_VEL_MAX_ACC_XY/255.0f);
	pOutput->limits.accMaxW = ((float)pVel->accMaxW)*(LOCAL_VEL_MAX_ACC_W/255.0f);
	pOutput->limits.jerkMaxXY = ((float)pVel->jerkMaxXY)*(LOCAL_VEL_MAX_JERK_XY/255.0f);
	pOutput->limits.jerkMaxW = ((float)pVel->jerkMaxW)*(LOCAL_VEL_MAX_JERK_W/255.0f);

	pOutput->mode = CTRL_MODE_LOCAL_VEL;
	pOutput->updated = 1;
}

static void globalVelRun(void* pData, SkillOutput* pOutput)
{
	VelInput* pVel = (VelInput*)pData;

	pOutput->xyw[0] = pVel->xyw[0]*0.001f;
	pOutput->xyw[1] = pVel->xyw[1]*0.001f;
	pOutput->xyw[2] = pVel->xyw[2]*0.001f;

	if(pVel->accMaxXY == 0)
		pVel->accMaxXY = 1;

	if(pVel->accMaxW == 0)
		pVel->accMaxW = 1;

	if(pVel->jerkMaxXY == 0)
		pVel->jerkMaxXY = 1;

	if(pVel->jerkMaxW == 0)
		pVel->jerkMaxW = 1;

	pOutput->limits.accMaxXY = ((float)pVel->accMaxXY)*(LOCAL_VEL_MAX_ACC_XY/255.0f);
	pOutput->limits.accMaxW = ((float)pVel->accMaxW)*(LOCAL_VEL_MAX_ACC_W/255.0f);
	pOutput->limits.jerkMaxXY = ((float)pVel->jerkMaxXY)*(LOCAL_VEL_MAX_JERK_XY/255.0f);
	pOutput->limits.jerkMaxW = ((float)pVel->jerkMaxW)*(LOCAL_VEL_MAX_JERK_W/255.0f);

	CtrlUtilTurnGlobal2Local(ctrl.state.pos[2], pOutput->xyw[0], pOutput->xyw[1], pOutput->xyw, pOutput->xyw+1);

	pOutput->mode = CTRL_MODE_LOCAL_VEL;
	pOutput->updated = 1;
}

static void globalPosRun(void* pData, SkillOutput* pOutput)
{
	GlobalPosInput* pPos = (GlobalPosInput*)pData;

	pOutput->xyw[0] = pPos->xyw[0]*0.001f;
	pOutput->xyw[1] = pPos->xyw[1]*0.001f;
	pOutput->xyw[2] = pPos->xyw[2]*0.001f;

	if(pPos->velMaxXY == 0)
		pPos->velMaxXY = 1;

	if(pPos->velMaxW == 0)
		pPos->velMaxW = 1;

	if(pPos->accMaxXY == 0)
		pPos->accMaxXY = 1;

	if(pPos->accMaxW == 0)
		pPos->accMaxW = 1;

	pOutput->limits.velMaxXY = ((float)pPos->velMaxXY)*(GLOBAL_POS_MAX_VEL_XY/255.0f);
	pOutput->limits.velMaxW = ((float)pPos->velMaxW)*(GLOBAL_POS_MAX_VEL_W/255.0f);
	pOutput->limits.accMaxXY = ((float)pPos->accMaxXY)*(GLOBAL_POS_MAX_ACC_XY/255.0f);
	pOutput->limits.accMaxW = ((float)pPos->accMaxW)*(GLOBAL_POS_MAX_ACC_W/255.0f);

	pOutput->mode = CTRL_MODE_GLOBAL_POS;
	pOutput->updated = 1;
}

static void wheelVelRun(void* pData, SkillOutput* pOutput)
{
	int16_t* pMotorVel = (int16_t*)pData;

	for(uint8_t i = 0; i < 4; i++)
	{
		pOutput->xyw[i] = pMotorVel[i]*0.005f;
	}

	pOutput->mode = CTRL_MODE_WHEEL_VEL;
	pOutput->updated = 1;
}
