/*
 * log_file.h
 *
 *  Created on: 24.08.2014
 *      Author: AndreR
 */

#ifndef LOG_FILE_H_
#define LOG_FILE_H_

#include <stdint.h>
#include "ch.h"
#include "util/fifo_block.h"
#include "fatfs/ff.h"
#include "config.h"

// Internals
#define LOGFILE_EVENT_START 0
#define LOGFILE_EVENT_UPDATE 1
#define LOGFILE_EVENT_CLOSE 2

typedef struct _LogFileEvent
{
	uint8_t eventType;
	const char* pFilename;
	uint32_t fileSize;
	uint32_t bytesPerSecIn;
	uint32_t writeSpeed;	// B/s
} LogFileEvent;

typedef void(*LogFileCb)(LogFileEvent*);

#define LOG_FILE_QUEUE_SIZE	10
#define LOG_FILE_WORKING_BUF_SIZE 512

typedef struct _LogFile
{
	FIL file;
	uint8_t ready;
	uint8_t writeFail;
	uint32_t fileSize;

	FifoBlock writeFifo;

	msg_t eventQueueData[LOG_FILE_QUEUE_SIZE];
	mailbox_t eventQueue;

	char logFilename[64];
	uint8_t cobsEncodeBuf[LOG_FILE_WORKING_BUF_SIZE];
	uint8_t initBuf[LOG_FILE_WORKING_BUF_SIZE];
	mutex_t encodeMtx;

	LogFileEvent event;
	LogFileCb eventCb;

	uint32_t bytesPushed;	// short term
	uint32_t lastPushedTime;
} LogFile;

extern LogFile logFile;

void	LogFileInit(uint8_t* pWriteBuf, uint32_t numChunks, uint32_t chunkSize);
void	LogFileOpen(const char* pName);
void	LogFileClose();
void	LogFileTask(void* params);
void	LogFileWrite(void* pLogMsg);

int16_t LogFilePackConfigDesc(const ConfigFileDesc* pCfg, uint8_t* pOut, uint32_t outSize, uint32_t* pBytesWritten);
int16_t LogFilePackConfigFile(const ConfigFile* pFile, uint8_t* pOut, uint32_t outSize, uint32_t* pBytesWritten);

#endif /* LOG_FILE_H_ */
