/*
 * prog_store.h
 *
 *  Created on: 01.04.2015
 *      Author: AndreR
 */

#ifndef PROG_STORE_H_
#define PROG_STORE_H_

#include "ch.h"
#include "intercom_constants.h"
#include <stdint.h>

#define PROG_STORE_ID_MAIN		0
#define PROG_STORE_ID_MEDIA		1
#define PROG_STORE_ID_LEFT		2
#define PROG_STORE_ID_RIGHT		3
#define PROG_STORE_ID_KD		4
#define PROG_STORE_ID_MAIN2016	5

#define PROG_STORE_MAX_SOURCES 2
#define PROG_STORE_MAX_TARGETS 5

#define PROG_STORE_ERROR_TIMEOUT	1
#define PROG_STORE_ERROR_NO_MEDIA	2

#define PROG_STORE_SOURCE_WIFI		0
#define PROG_STORE_SOURCE_USB		1

#define PROG_STORE_MAX_CHUNK_SIZE 256

typedef void(*ProgSourceRequestDataFunc)(uint8_t procId, uint32_t offset, uint32_t size);
typedef void(*ProgSourceRequestSizeFunc)(uint8_t procId);
typedef void(*ProgSourceRequestCRC32)(uint8_t procId, uint32_t startAddr, uint32_t endAddr);
typedef void(*ProgSourceOpenClose)();

typedef struct _ProgSource
{
	ProgSourceRequestDataFunc pDataFunc;
	ProgSourceRequestSizeFunc pSizeFunc;
	ProgSourceRequestCRC32 pCRC32Func;
	ProgSourceOpenClose pOpenFunc;
	ProgSourceOpenClose pCloseFunc;

	uint16_t timeout; // [ms]
	uint16_t maxTransferSize; // [B]
} ProgSource;

typedef struct _ProgStoreEvent
{
	uint8_t procId;
	uint16_t type;

	union
	{
		int16_t errorId;

		struct _data
		{
			uint32_t offset;
			uint32_t size;
		} data;

		uint32_t size;

		struct _crc32
		{
			uint32_t startAddr;
			uint32_t endAddr;
			uint32_t crc;
		} crc32;
	};
} ProgStoreEvent;

typedef struct _ProgTarget
{
	uint8_t intercomAddress;
	uint32_t flashSize;
	uint8_t programId;
} ProgTarget;

typedef struct PACKED _ProgUpgadeInfo
{
	uint32_t update;
	uint32_t crc;
	uint32_t size;
} ProgUpgradeInfo;

typedef struct _ProgStore
{
	ProgSource sources[PROG_STORE_MAX_SOURCES];
	uint8_t usedSources;
	uint8_t useSource;

	ProgStoreEvent lastEvent;
	binary_semaphore_t storeSem;

	ProgTarget targets[PROG_STORE_MAX_TARGETS];
	uint8_t usedTargets;

	uint8_t progDataBuf[PROG_STORE_MAX_CHUNK_SIZE+sizeof(BootloaderProgramData)];

	uint16_t progSeq;
	uint16_t expectedSeq;
	volatile uint8_t progAckd;
	binary_semaphore_t progSem;

	thread_t* pProgThread;
} ProgStore;

extern ProgStore progStore;

void ProgStoreInit();
void ProgStoreStart(uint8_t source);
void ProgStoreFlash();
int16_t ProgStoreAddSource(const ProgSource* pSource);
int16_t ProgStoreAddTarget(uint8_t address, uint32_t flashSize, uint8_t programId);

// ProgSource interface
void ProgStoreProvideError(uint8_t procId, int16_t error);
void ProgStoreProvideData(uint8_t procId, uint32_t offset, uint8_t* pData, uint32_t size);
void ProgStoreProvideSize(uint8_t procId, uint32_t size);
void ProgStoreProvideCRC32(uint8_t procId, uint32_t startAddr, uint32_t endAddr, uint32_t crc32);

// Implementation interface
void ProgStoreImplInit();
void ProgStoreImplGetUpgradeInfo(uint8_t procId, ProgUpgradeInfo* pInfo);
void ProgStoreImplSetUpgradeInfo(uint8_t procId, const ProgUpgradeInfo* pInfo);
void ProgStoreImplReadData(uint8_t procId, uint32_t offset, uint8_t* pBuf, uint32_t size);
void ProgStoreImplWriteData(uint8_t procId, uint32_t offset, uint8_t* pData, uint32_t size);
uint32_t ProgStoreImplGetChecksum(uint8_t procId, uint32_t offset, uint32_t size);
void ProgStoreImplEraseStorage();

#endif /* PROG_STORE_H_ */
